@@ -20,6 +20,9 @@ LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE. */
 
+#include "Preferences.hpp"
+
+
 
 #include "stdafx.hpp"
 #include "OptionsDialog.hpp"
@@ -42,12 +45,14 @@ THE SOFTWARE. */
 MainWindow::MainWindow() :
 		isFullscreen(false),
 		optionsMenuItem("Options"),
+		saveTuningMenuItem("Save tuning"),
 		exitMenuItem("Exit"),
 		aboutMenuItem("About"),
 		optionsDialog(*this),
 		aboutDialog(*this),
-		numOctaves(1),
-		basePitch(12 * 4),//A4
+		prefs(MainWindow::DPreferencesFileName()),
+//		numOctaves(1),
+//		basePitch(12 * 4),//A4
 		tappedX(0)
 {
 	this->add_events(
@@ -87,6 +92,14 @@ MainWindow::MainWindow() :
 			sigc::mem_fun(*this, &MainWindow::ShowOptionsDialog)
 		);
 
+	this->saveTuningMenuItem.set_size_request(-1, DFingerFriendlyMinSize());
+	this->mainMenu.append(this->saveTuningMenuItem);
+	this->saveTuningMenuItem.signal_activate().connect(
+			sigc::mem_fun(*this, &MainWindow::SavePreferencesToFile)
+		);
+
+	this->mainMenu.append(this->menuSeparator2);
+
 	this->exitMenuItem.set_size_request(-1, DFingerFriendlyMinSize());
 	this->mainMenu.append(this->exitMenuItem);
 	this->exitMenuItem.signal_activate().connect(
@@ -102,12 +115,12 @@ MainWindow::MainWindow() :
 		);
 
 	this->mainMenu.show_all();//Show the main menu and all its child widgets.
-	this->set_main_menu(this->mainMenu);
+	this->set_menu(this->mainMenu);
 
 	this->show_all_children();//Show all window child widgets
 
 	//set default timbre
-	this->ApplyTimbre(OptionsDialog::SINE_PLUS_ADSR);
+	this->ApplyTimbre(Preferences::SINE_PLUS_ADSR);
 
 //	TRACE(<< "MainWindow::ShowOptionsDialog(): loading preferences" << std::endl)
 	this->LoadPreferencesFromFile();
@@ -121,14 +134,6 @@ void MainWindow::Exit(){
 
 
 
-void MainWindow::ClampBasePitch(){
-	if(this->basePitch + 12 * this->numOctaves > (DHighestOctave() + 1) * 12){
-		this->basePitch = (DHighestOctave() - (this->numOctaves)) * 12;
-	}
-}
-
-
-
 void MainWindow::ShowAboutDialog(){
 	this->aboutDialog.run();
 	this->aboutDialog.hide();
@@ -138,24 +143,25 @@ void MainWindow::ShowAboutDialog(){
 
 void MainWindow::ShowOptionsDialog(){
 //	TRACE(<< "MainWindow::ShowOptionsDialog(): enter" << std::endl)
-	this->optionsDialog.SetNumOctaves(this->numOctaves);
-	this->optionsDialog.SetTimbre(this->GetCurrentTimbreType());
+	this->optionsDialog.SetNumOctaves(this->prefs.GetNumOctaves());
+	this->optionsDialog.SetTimbre(this->prefs.GetTimbre());
+	this->optionsDialog.SetLockToTappedOctave(this->prefs.GetLockCurrentOctave());
 
 	switch(this->optionsDialog.run()){
 		case 0: //OK
-			this->numOctaves = this->optionsDialog.GetNumOctaves();
-			this->ApplyTimbre(this->optionsDialog.GetTimbre());
+			this->prefs.SetNumOctaves(this->optionsDialog.GetNumOctaves());
+			this->prefs.SetTimbre(this->optionsDialog.GetTimbre());
+			this->prefs.SetLockCurrentOctave(this->optionsDialog.GetLockToTappedOctave());
+			this->ApplyTimbre(this->prefs.GetTimbre());
+			
+			this->SavePreferencesToFile();
 			break;
 		default: //Cancel and all other cases
-			this->optionsDialog.SetNumOctaves(this->numOctaves);
-			this->optionsDialog.SetTimbre(this->GetCurrentTimbreType());
+			this->optionsDialog.SetNumOctaves(this->prefs.GetNumOctaves());
+			this->optionsDialog.SetTimbre(this->prefs.GetTimbre());
+			this->optionsDialog.SetLockToTappedOctave(this->prefs.GetLockCurrentOctave());
 			break;
 	}
-
-	//adjust base pitch
-	this->ClampBasePitch();
-
-	this->SavePreferencesToFile();
 	
 	this->optionsDialog.hide();
 	this->queue_draw();//request redrawing entire window
@@ -164,13 +170,13 @@ void MainWindow::ShowOptionsDialog(){
 
 
 
-void MainWindow::ApplyTimbre(OptionsDialog::E_Timbre timbre){
+void MainWindow::ApplyTimbre(Preferences::E_Timbre timbre){
 	switch(timbre){
 		default:
-		case OptionsDialog::SINE_PLUS_ADSR:
+		case Preferences::SINE_PLUS_ADSR:
 			this->instr.Seize(new aumiks::SineWaveInstrument());
 			break;
-		case OptionsDialog::PLUCKED_STRING:
+		case Preferences::PLUCKED_STRING:
 			this->instr.Seize(new aumiks::PluckedStringInstrument());
 			break;
 	}
@@ -178,11 +184,11 @@ void MainWindow::ApplyTimbre(OptionsDialog::E_Timbre timbre){
 
 
 
-OptionsDialog::E_Timbre MainWindow::GetCurrentTimbreType()const{
+Preferences::E_Timbre MainWindow::GetCurrentTimbreType()const{
 	if(this->instr.DynamicCast<aumiks::PluckedStringInstrument>().IsValid()){
-		return OptionsDialog::PLUCKED_STRING;
+		return Preferences::PLUCKED_STRING;
 	}else{
-		return OptionsDialog::SINE_PLUS_ADSR;
+		return Preferences::SINE_PLUS_ADSR;
 	}
 }
 
@@ -214,14 +220,14 @@ bool MainWindow::OnExposeEvent(GdkEventExpose* event){
 	c->select_font_face("Sans", Cairo::FONT_SLANT_NORMAL, Cairo::FONT_WEIGHT_NORMAL);
 	c->set_font_size(50.0);
 
-	for(ting::uint i = 0; i < this->numOctaves; ++i){
+	for(ting::uint i = 0; i < this->prefs.GetNumOctaves(); ++i){
 		c->save();
-		c->translate(0, (i * height) / this->numOctaves);
+		c->translate(0, (i * height) / this->prefs.GetNumOctaves());
 		MainWindow::DrawOctave(
 				c,
 				width,
-				height / this->numOctaves,
-				this->basePitch + (this->numOctaves - 1 - i) * 12
+				height / this->prefs.GetNumOctaves(),
+				this->prefs.GetBasePitch() + (this->prefs.GetNumOctaves() - 1 - i) * 12
 			);
 		c->restore();
 	}
@@ -338,26 +344,37 @@ void MainWindow::OnKeyPress_Notify(GdkEventKey* event){
 	switch(event->keyval){
 		case GDK_Up:
 			//DHighestOctave() + 1 is the number of octaves
-			if(this->basePitch + this->PitchRange() >= DHighestOctave() * 12)
-				this->basePitch = (DHighestOctave() + 1 - this->numOctaves) * 12;
-			else
-				this->basePitch += 12;
+			if(this->prefs.GetBasePitch() + this->PitchRange() >= DHighestOctave() * 12){
+				this->prefs.SetBasePitch(
+						(DHighestOctave() + 1 - this->prefs.GetNumOctaves()) * 12
+					);
+			}else{
+				this->prefs.SetBasePitch(
+						this->prefs.GetBasePitch() + 12
+					);
+			}
 
 			this->queue_draw();//request redrawing etire window
 			break;
 
 		case GDK_Down:
-			if(this->basePitch <= 12)
-				this->basePitch = 0;
-			else
-				this->basePitch -= 12;
+			if(this->prefs.GetBasePitch() <= 12){
+				this->prefs.SetBasePitch(0);
+			}else{
+				this->prefs.SetBasePitch(
+						this->prefs.GetBasePitch() - 12
+					);
+			}
 
 			this->queue_draw();//request redrawing etire window
 			break;
 
 		case GDK_Left:
-			if(this->basePitch != 0)
-				--this->basePitch;
+			if(this->prefs.GetBasePitch() != 0){
+				this->prefs.SetBasePitch(
+						this->prefs.GetBasePitch() - 1
+					);
+			}
 
 			this->queue_draw();//request redrawing etire window
 			break;
@@ -365,8 +382,11 @@ void MainWindow::OnKeyPress_Notify(GdkEventKey* event){
 		case GDK_Right:
 			//octaves index starts from 0, here need the number of octaves,
 			//i.e. DHighestOctave() + 1
-			if(this->basePitch + this->PitchRange() < 12 * (DHighestOctave() + 1))
-				++this->basePitch;
+			if(this->prefs.GetBasePitch() + this->PitchRange() < 12 * (DHighestOctave() + 1)){
+				this->prefs.SetBasePitch(
+						this->prefs.GetBasePitch() + 1
+					);
+			}
 
 			this->queue_draw();//request redrawing etire window
 			break;
@@ -405,27 +425,31 @@ MainWindow::FreqVol MainWindow::CalcFreqAndVol(
 	FreqVol ret;
 	x -= (width / 12) / 2;
 
-	float octHeight = height / float(this->numOctaves);
+	float octHeight = height / float(this->prefs.GetNumOctaves());
 
 	//find the screen octave number of the touched octave
 	ting::uint touchedOctave = 0;
-	for(ting::uint i = 1; i <= this->numOctaves; ++i){
+	for(ting::uint i = 1; i <= this->prefs.GetNumOctaves(); ++i){
 		if(y < i * octHeight){
 			touchedOctave = i;
 			break;
 		}
 	}
 
-	//if press event then update current playing octave, this is to prevent volume leaps
-	if(isPressEvent)
+	if(this->prefs.GetLockCurrentOctave()){
+		//if press event then update current playing octave, this is to prevent volume leaps
+		if(isPressEvent)
+			this->curOctave = touchedOctave;
+	}else{
 		this->curOctave = touchedOctave;
+	}
 
 	float pow = 1;
-	for(ting::uint j = 0; j < (this->numOctaves - this->curOctave); ++j)
+	for(ting::uint j = 0; j < (this->prefs.GetNumOctaves() - this->curOctave); ++j)
 		pow *= 2;
 
 	ret.freq = MainWindow::CalcFreq(
-			MainWindow::PitchToFreq(this->basePitch) * pow,
+			MainWindow::PitchToFreq(this->prefs.GetBasePitch()) * pow,
 			x / width
 		);
 
@@ -519,104 +543,13 @@ bool MainWindow::OnMotionEvent(GdkEventMotion* event){
 
 
 void MainWindow::LoadPreferencesFromFile(){
-//	TRACE(<< "MainWindow::LoadPreferencesFromFile(): enter" << std::endl)
+	this->prefs.LoadFromFile();
 
-	file::FSFile f(file::File::GetHomeDir() + ".theremin/" + MainWindow::DPreferencesFileName());
-
-//	TRACE(<< "MainWindow::LoadPreferencesFromFile(): reading file" << std::endl)
-
-	pugi::xml_document doc;
-	{
-		ting::Array<ting::byte> rawFile;
-		try{
-			rawFile = f.LoadWholeFileIntoMemory();
-		}catch(file::File::Exc &e){
-			LOG(
-					<< "MainWindow::LoadPreferencesFromFile(): could not load preferences file." << std::endl
-					<< "\t" << " e = " << e.What() << std::endl
-				)
-			return;
-		}
-
-		doc.parse(reinterpret_cast<char*>(rawFile.Buf()));
-	}
-
-	{
-		pugi::xml_node opts = doc.child("Options");
-		if(!opts.empty()){
-			pugi::xml_attribute numOctavesAttr = opts.attribute("numOctaves");
-			if(!numOctavesAttr.empty()){
-				ting::uint num = numOctavesAttr.as_uint();
-				if(num != 0 && num <= 3)
-					this->numOctaves = num;
-			}
-
-			pugi::xml_attribute timbreAttr = opts.attribute("timbre");
-			if(!timbreAttr.empty()){
-				this->ApplyTimbre(
-						OptionsDialog::TimbreNameToType(timbreAttr.value())
-					);
-			}
-		}
-	}
-
-	{
-		pugi::xml_node tun = doc.child("Tuning");
-		if(!tun.empty()){
-			pugi::xml_attribute basePitchAttr = tun.attribute("basePitch");
-			if(!basePitchAttr.empty()){
-				this->basePitch = basePitchAttr.as_uint();
-				this->ClampBasePitch();
-			}
-		}
-		TRACE(<< "Load prefs, new basePitch = " << this->basePitch << std::endl)
-	}
+	this->ApplyTimbre(this->prefs.GetTimbre());
 }
 
 
 
 void MainWindow::SavePreferencesToFile(){
-	pugi::xml_document doc;
-	
-	{
-		pugi::xml_node n = doc.append_child(pugi::node_declaration);
-		n.set_name("xml");
-		n.append_attribute("version").set_value("1.0");
-	}
-
-	//TODO: move tag names to separate file
-	{
-		pugi::xml_node opts = doc.append_child();
-		opts.set_name("Options");
-		opts.append_attribute("numOctaves") = this->numOctaves;
-		opts.append_attribute("timbre").set_value(
-				OptionsDialog::TimbreTypeToName(
-						this->GetCurrentTimbreType()
-					).c_str()
-			);
-	}
-
-	{
-		pugi::xml_node tun = doc.append_child();
-		tun.set_name("Tuning");
-		tun.append_attribute("basePitch") = this->basePitch;
-	}
-
-	class PrefsWriter : public pugi::xml_writer{
-		file::FSFile f;
-	public:
-		PrefsWriter() :
-				f(file::File::GetHomeDir() + ".theremin/" + MainWindow::DPreferencesFileName())
-		{
-			f.Open(file::File::CREATE);
-		}
-
-		//override
-		void write(const void *data, size_t size){
-			ting::Array<ting::byte> buf(size);
-			memcpy(buf.Buf(), data, size);
-			f.Write(buf);
-		}
-	} writer;
-    doc.print(writer);
+	this->prefs.SaveToFile();
 }
