@@ -30,6 +30,7 @@ THE SOFTWARE. */
 #include "OptionsDialog.hpp"
 
 
+
 OptionsDialog::OptionsDialog(Gtk::Window &parent) :
 		Gtk::Dialog("Options", parent, true),
 		octavesFrame("Number of octaves"),
@@ -37,8 +38,9 @@ OptionsDialog::OptionsDialog(Gtk::Window &parent) :
 		twoOctavesRadioButton("Two octaves"),
 		threeOctavesRadioButton("Three octaves"),
 		timbreFrame("Timbre"),
-		sinePlusADSRButton("Sine + ADSR"),
-		pluckedStringButton("Plucked string")
+		sinePlusADSRRadioButton("Sine + ADSR"),
+		pluckedStringRadioButton("Plucked string"),
+		lockCurrentOctaveCheckbox("Lock to tapped octave")
 {
 	this->get_vbox()->add(this->hBox);
 	this->hBox.add(this->octavesFrame);
@@ -61,20 +63,24 @@ OptionsDialog::OptionsDialog(Gtk::Window &parent) :
 	this->timbreFrame.add(this->timbreBox);
 
 	{//group timbre radio buttons
-		Gtk::RadioButtonGroup gr = this->sinePlusADSRButton.get_group();
-		this->pluckedStringButton.set_group(gr);
+		Gtk::RadioButtonGroup gr = this->sinePlusADSRRadioButton.get_group();
+		this->pluckedStringRadioButton.set_group(gr);
 	}
-	this->timbreBox.add(this->sinePlusADSRButton);
-	this->sinePlusADSRButton.set_size_request(-1, DFingerFriendlyMinSize());
-	this->timbreBox.add(this->pluckedStringButton);
-	this->pluckedStringButton.set_size_request(-1, DFingerFriendlyMinSize());
+	this->timbreBox.add(this->sinePlusADSRRadioButton);
+	this->sinePlusADSRRadioButton.set_size_request(-1, DFingerFriendlyMinSize());
+	this->timbreBox.add(this->pluckedStringRadioButton);
+	this->pluckedStringRadioButton.set_size_request(-1, DFingerFriendlyMinSize());
 
+	this->lockCurrentOctaveCheckbox.set_size_request(-1, DFingerFriendlyMinSize());
+	this->get_vbox()->add(this->lockCurrentOctaveCheckbox);
 
 	this->add_button("Ok", 0);
 	this->add_button("Cancel", 1);
 
 	this->show_all_children();//Show all window child widgets
-};
+}
+
+
 
 ting::uint OptionsDialog::GetNumOctaves()const{
 	if(this->oneOctaveRadioButton.get_active())
@@ -85,7 +91,9 @@ ting::uint OptionsDialog::GetNumOctaves()const{
 		return 3;
 	else
 		return 0;
-};
+}
+
+
 
 void OptionsDialog::SetNumOctaves(ting::uint num){
 	switch(num){
@@ -100,48 +108,42 @@ void OptionsDialog::SetNumOctaves(ting::uint num){
 		default:
 			break;
 	}
-};
+}
 
-OptionsDialog::E_Timbre OptionsDialog::GetTimbre()const{
-	if(this->sinePlusADSRButton.get_active())
-		return SINE_PLUS_ADSR;
-	else if(this->pluckedStringButton.get_active())
-		return PLUCKED_STRING;
+
+
+Preferences::E_Timbre OptionsDialog::GetTimbre()const{
+	if(this->sinePlusADSRRadioButton.get_active())
+		return Preferences::SINE_PLUS_ADSR;
+	else if(this->pluckedStringRadioButton.get_active())
+		return Preferences::PLUCKED_STRING;
 	else
-		return UNKNOWN;
+		return Preferences::UNKNOWN;
 }
 
-void OptionsDialog::SetTimbre(E_Timbre timbre){
+
+
+void OptionsDialog::SetTimbre(Preferences::E_Timbre timbre){
 	switch(timbre){
-		case SINE_PLUS_ADSR:
-			this->sinePlusADSRButton.set_active(true);
+		case Preferences::SINE_PLUS_ADSR:
+			this->sinePlusADSRRadioButton.set_active(true);
 			break;
-		case PLUCKED_STRING:
-			this->pluckedStringButton.set_active(true);
+		case Preferences::PLUCKED_STRING:
+			this->pluckedStringRadioButton.set_active(true);
 			break;
 		default:
 			break;
 	}
 }
 
-//static
-std::string OptionsDialog::TimbreTypeToName(E_Timbre timbre){
-	switch(timbre){
-		default:
-		case SINE_PLUS_ADSR:
-			return "SineWave";
-			break;
-		case PLUCKED_STRING:
-			return "PluckedString";
-			break;
-	}
+
+
+bool OptionsDialog::GetLockToTappedOctave()const{
+	return this->lockCurrentOctaveCheckbox.get_active();
 }
 
-//static
-OptionsDialog::E_Timbre OptionsDialog::TimbreNameToType(std::string name){
-	for(ting::uint t = 0; t < ting::uint(NUM_TIMBRES); ++t){
-		if(OptionsDialog::TimbreTypeToName(E_Timbre(t)).compare(name) == 0)
-			return E_Timbre(t);
-	}
-	return SINE_PLUS_ADSR;
+
+
+void OptionsDialog::SetLockToTappedOctave(bool lockToOctave){
+	this->lockCurrentOctaveCheckbox.set_active(lockToOctave);
 }
