@@ -34,9 +34,12 @@ THE SOFTWARE. */
 
 #include <gtkmm/dialog.h>
 #include <gtkmm/radiobutton.h>
+#include <gtkmm/checkbutton.h>
 #include <gtkmm/box.h>
 #include <gtkmm/frame.h>
 
+#include "Preferences.hpp"
+
 class OptionsDialog : public Gtk::Dialog{
 	Gtk::HBox hBox;
 
@@ -48,28 +51,21 @@ class OptionsDialog : public Gtk::Dialog{
 
 	Gtk::Frame timbreFrame;
 	Gtk::VBox timbreBox;
-	Gtk::RadioButton sinePlusADSRButton;
-	Gtk::RadioButton pluckedStringButton;
+	Gtk::RadioButton sinePlusADSRRadioButton;
+	Gtk::RadioButton pluckedStringRadioButton;
+
+	Gtk::CheckButton lockCurrentOctaveCheckbox;
 public:
 	OptionsDialog(Gtk::Window &parent);
 
 	ting::uint GetNumOctaves()const;
 	void SetNumOctaves(ting::uint num);
 
-	enum E_Timbre{
-		SINE_PLUS_ADSR,
-		PLUCKED_STRING,
-		
-		UNKNOWN, //NOTE: UNKNOWN must be last timbre in the list!!!
-		
-		NUM_TIMBRES
-	};
+	Preferences::E_Timbre GetTimbre()const;
+	void SetTimbre(Preferences::E_Timbre timbre);
 
-	static std::string TimbreTypeToName(E_Timbre timbre);
-	static E_Timbre TimbreNameToType(std::string name);
-
-	E_Timbre GetTimbre()const;
-	void SetTimbre(E_Timbre timbre);
+	bool GetLockToTappedOctave()const;
+	void SetLockToTappedOctave(bool lockToOctave);
 };
 
 #endif//~once
