@@ -32,12 +32,20 @@ THE SOFTWARE. */
 
 #include <cmath>
 #include <ting/math.hpp>
+#include <ting/types.hpp>
 
 using namespace aumiks;
 
 
 
 bool SineWaveInstrument::Channel::MixToMixBuf(Array<s32>& mixBuf){
+	if(this->lastVolume < 0){
+		this->lastVolume = this->volume;
+	}
+	int startVolume = this->lastVolume;
+	int endVolume = this->volume;
+	this->lastVolume = this->volume;
+
 	bool stopPlaying = false;
 	bool quit;
 	ting::uint i = 0;//current sample
@@ -52,8 +60,11 @@ bool SineWaveInstrument::Channel::MixToMixBuf(Array<s32>& mixBuf){
 					this->curPhase += 2 * DPi<float>() * this->frequency * (1.0f / float(44100));
 					this->curTime += 1.0f / float(44100);
 
+					//interpolate volume
+					ting::u8 vol = ting::u8(startVolume + ((endVolume - startVolume) * i) / mixBuf.Size());
+
 					//apply volume
-					smp = (smp * this->volume) / 0xff;
+					smp = (smp * vol) / 0xff;
 
 					mixBuf[i] += smp;
 					++i;
@@ -77,8 +88,11 @@ bool SineWaveInstrument::Channel::MixToMixBuf(Array<s32>& mixBuf){
 					this->curPhase += 2 * DPi<float>() * this->frequency * (1.0f / float(44100));
 					this->curTime += 1.0f / float(44100);
 
+					//interpolate volume
+					ting::u8 vol = ting::u8(startVolume + ((endVolume - startVolume) * i) / mixBuf.Size());
+
 					//apply volume
-					smp = (smp * this->volume) / 0xff;
+					smp = (smp * vol) / 0xff;
 
 					mixBuf[i] += smp;
 					++i;
@@ -110,8 +124,11 @@ bool SineWaveInstrument::Channel::MixToMixBuf(Array<s32>& mixBuf){
 				//		TRACE(<<"t =" << this->t)
 				//		TRACE(<<"smp =" << smp)
 
+						//interpolate volume
+						ting::u8 vol = ting::u8(startVolume + ((endVolume - startVolume) * i) / mixBuf.Size());
+
 						//apply volume
-						smp = (smp * this->volume) / 0xff;
+						smp = (smp * vol) / 0xff;
 
 						mixBuf[i] += smp;
 						++i;
@@ -119,6 +136,12 @@ bool SineWaveInstrument::Channel::MixToMixBuf(Array<s32>& mixBuf){
 						++i;
 					}//~for
 				}
+
+				{
+					float numPeriods = this->curPhase / (2 * DPi<float>());
+					this->curPhase -= float(ting::uint(numPeriods)) * 2 * DPi<float>();
+				}
+
 				break;//~switch
 			case RELEASE:
 				for(; i < mixBuf.Size();){
@@ -127,8 +150,11 @@ bool SineWaveInstrument::Channel::MixToMixBuf(Array<s32>& mixBuf){
 					this->curPhase += 2 * DPi<float>() * this->frequency * (1.0f / float(44100));
 					this->curTime += 1.0f / float(44100);
 
+					//interpolate volume
+					ting::u8 vol = ting::u8(startVolume + ((endVolume - startVolume) * i) / mixBuf.Size());
+
 					//apply volume
-					smp = (smp * this->volume) / 0xff;
+					smp = (smp * vol) / 0xff;
 
 					mixBuf[i] += smp;
 					++i;
@@ -166,6 +192,13 @@ Ref<Instrument::Channel> PluckedStringInstrument::CreateChannel(){
 }
 
 bool PluckedStringInstrument::Channel::MixToMixBuf(Array<s32>& mixBuf){
+	if(this->lastVolume < 0){
+		this->lastVolume = this->volume;
+	}
+	int startVolume = this->lastVolume;
+	int endVolume = this->volume;
+	this->lastVolume = this->volume;
+
 	bool ret = false;
 
 	switch(this->state){
@@ -194,8 +227,12 @@ bool PluckedStringInstrument::Channel::MixToMixBuf(Array<s32>& mixBuf){
 		//with distortion
 //		s32 smp = s32( 0x7fff * (s > 0 ? tanh(4 * s) : s ) );
 
+		//interpolate volume
+		ting::u8 vol = ting::u8(startVolume + ((endVolume - startVolume) * i) / mixBuf.Size());
+
 		//apply volume
-		smp = (smp * this->volume) / 0xff;
+		smp = (smp * vol) / 0xff;
+
 
 		mixBuf[i] += smp;
 		++i;
