@@ -31,6 +31,7 @@ THE SOFTWARE. */
 #define M_Ref_hpp
 
 #include "debug.hpp"
+#include "types.hpp"
 
 //#define M_ENABLE_REF_PRINT
 #ifdef M_ENABLE_REF_PRINT
@@ -259,7 +260,11 @@ public:
 			return Ref<TS>();
 	}
 	
-	inline Ref() :
+	//the int argument is just to make possible
+	//auto conversion from 0 to invalid Ref object
+	//i.e. it will be possible to write 'return 0;'
+	//from the function returning Ref
+	inline Ref(int v = 0) :
 			p(0)
 	{
 		M_REF_PRINT(<<"Ref::Ref(): invoked, p="<<(this->p)<<std::endl)
@@ -359,31 +364,38 @@ public:
 		M_REF_PRINT(<<"Ref::operator*(): invoked, p="<<(this->p)<<std::endl)
 		ASSERT_INFO(this->p, "Ref::operator*(): this->p is zero")
 		return static_cast<T&>(*this->p);
-	};
+	}
 
 	inline const T& operator*()const{
 		M_REF_PRINT(<<"const Ref::operator*(): invoked, p="<<(this->p)<<std::endl)
 		ASSERT_INFO(this->p, "const Ref::operator*(): this->p is zero")
 		return static_cast<T&>(*this->p);
-	};
+	}
 
 	inline T* operator->(){
 		M_REF_PRINT(<<"Ref::operator->(): invoked, p="<<(this->p)<<std::endl)
 		ASSERT_INFO(this->p, "Ref::operator->(): this->p is zero")
 		return static_cast<T*>(this->p);
-	};
+	}
+
 
 	inline const T* operator->()const{
 		M_REF_PRINT(<<"Ref::operator->()const: invoked, p="<<(this->p)<<std::endl)
 		ASSERT_INFO(this->p, "Ref::operator->(): this->p is zero")
 		return static_cast<T*>(this->p);
-	};
+	}
 
 
 	//for type downcast
-	template <typename TBase> operator Ref<TBase>(){
-		return Ref<TBase>(this->operator->());
-	};
+	template <typename TBase> inline operator Ref<TBase>(){
+		//downcasting of invalid reference is also possible
+		if(this->IsNotValid())
+			return Ref<TBase>();
+		
+		//NOTE: static cast to T*, not to TBase*,
+		//this is to forbid automatic upcast
+		return Ref<TBase>(static_cast<T*>(this->p));
+	}
 
 
 private:
